@@ -4,27 +4,23 @@ use strict;
 use warnings;
 
 use IO::Async::Loop;
-use IO::Async::Stream;
+use IO::Async::Listener;
 
 my $PORT = 12345;
 
 my $loop = IO::Async::Loop->new;
 
-$loop->listen(
-   service  => $PORT,
-   socktype => 'stream',
+my $listener = IO::Async::Listener->new(
+   on_stream => sub {
+      my $self = shift;
+      my ( $stream ) = @_;
 
-   on_accept => sub {
-      my ( $socket ) = @_;
-
-      # $socket is just an IO::Socket reference
+      my $socket = $stream->read_handle;
       my $peeraddr = $socket->peerhost . ":" . $socket->peerport;
 
       print STDERR "Accepted new connection from $peeraddr\n";
 
-      my $clientstream = IO::Async::Stream->new(
-         handle => $socket,
-
+      $stream->configure(
          on_read => sub {
             my ( $self, $buffref, $closed ) = @_;
 
@@ -44,8 +40,15 @@ $loop->listen(
          },
       );
 
-      $loop->add( $clientstream );
+      $loop->add( $stream );
    },
+);
+
+$loop->add( $listener );
+
+$listener->listen(
+   service  => $PORT,
+   socktype => 'stream',
 
    on_resolve_error => sub { die "Cannot resolve - $_[0]\n"; },
    on_listen_error  => sub { die "Cannot listen\n"; },
