@@ -3,9 +3,11 @@
 #
 #  (C) Paul Evans, 2007 -- leonerd@leonerd.org.uk
 
-package IO::Async::DetachedCode::FlatMarshaller;
+package # hide from CPAN
+  IO::Async::DetachedCode::FlatMarshaller;
 
 use strict;
+use warnings;
 
 use Carp;
 
