@@ -8,7 +8,9 @@ package IO::Async::Loop;
 use strict;
 use warnings;
 
-our $VERSION = '0.33';
+our $VERSION = '0.34';
+
+# When editing this value don't forget to update the docs below
 use constant NEED_API_VERSION => '0.33';
 
 use Carp;
@@ -16,7 +18,7 @@ use Carp;
 use Socket;
 use IO::Socket;
 use Time::HiRes qw(); # empty import
-use POSIX qw( WNOHANG );
+use POSIX qw( _exit WNOHANG );
 
 # Try to load IO::Socket::INET6 but don't worry if we don't have it
 eval { require IO::Socket::INET6 };
@@ -581,26 +583,27 @@ sub later
 }
 
 # The following two methods are no longer needed; included just to keep legacy code happy
-sub enable_childmanager  { }
-sub disable_childmanager { }
+sub enable_childmanager
+{
+   carp "Loop->enable_childmanager is no longer needed; do not call it.";
+}
+
+sub disable_childmanager
+{
+   carp "Loop->disable_childmanager is no longer needed; do not call it.";
+}
 
 =head2 $pid = $loop->detach_child( %params )
 
-This method creates a new child process to run a given code block. For more
-detail, see the C<detach_child()> method on the L<IO::Async::ChildManager>
-class.
+This method creates a new child process to run a given code block. It is a
+legacy wrapper around the C<fork()> method.
 
 =cut
 
 sub detach_child
 {
    my $self = shift;
-   my %params = @_;
-
-   my $childmanager = $self->{childmanager} ||=
-      $self->__new_feature( "IO::Async::ChildManager" );
-
-   $childmanager->detach_child( %params );
+   $self->fork( @_ );
 }
 
 =head2 $code = $loop->detach_code( %params )
@@ -627,7 +630,7 @@ sub detach_code
 =head2 $loop->spawn_child( %params )
 
 This method creates a new child process to run a given code block or command.
-For more detail, see the C<detach_child()> method on the
+For more detail, see the C<spawn_child()> method on the
 L<IO::Async::ChildManager> class.
 
 =cut
@@ -682,6 +685,19 @@ sub run_child
    $childmanager->run_child( %params );
 }
 
+=head2 $loop->resolver
+
+Returns the internally-stored L<IO::Async::Resolver> object, used for name
+resolution operations by the C<resolve>, C<connect> and C<listen> methods.
+
+=cut
+
+sub resolver
+{
+   my $self = shift;
+   return $self->{resolver} ||= $self->__new_feature( "IO::Async::Resolver" );
+}
+
 =head2 $loop->resolve( %params )
 
 This method performs a single name resolution operation. It uses an
@@ -695,9 +711,7 @@ sub resolve
    my $self = shift;
    my ( %params ) = @_;
 
-   my $resolver = $self->{resolver} ||= $self->__new_feature( "IO::Async::Resolver" );
-
-   $resolver->resolve( %params );
+   $self->resolver->resolve( %params );
 }
 
 =head2 $loop->connect( %params )
@@ -1031,6 +1045,76 @@ sub time
    return Time::HiRes::time();
 }
 
+=head2 $pid = $loop->fork( %params )
+
+This method creates a new child process to run a given code block, returning
+its process ID.
+
+=over 8
+
+=item code => CODE
+
+A block of code to execute in the child process. It will be called in scalar
+context inside an C<eval> block. The return value will be used as the
+C<exit()> code from the child if it returns (or 255 if it returned C<undef> or
+thows an exception).
+
+=item on_exit => CODE
+
+A optional continuation to be called when the child processes exits. It will
+be invoked in the following way:
+
+ $on_exit->( $pid, $exitcode )
+
+The second argument is passed the plain perl C<$?> value. To use that
+usefully, see C<WEXITSTATUS()> and others from C<POSIX>.
+
+This key is optional; if not supplied, the calling code should install a
+handler using the C<watch_child()> method.
+
+=item keep_signals => BOOL
+
+Optional boolean. If missing or false, any CODE references in the C<%SIG> hash
+will be removed and restored back to C<DEFAULT> in the child process. If true,
+no adjustment of the C<%SIG> hash will be performed.
+
+=back
+
+=cut
+
+sub fork
+{
+   my $self = shift;
+   my %params = @_;
+
+   my $code = $params{code} or croak "Expected 'code' as a CODE reference";
+
+   my $kid = fork();
+   defined $kid or croak "Cannot fork() - $!";
+
+   if( $kid == 0 ) {
+      unless( $params{keep_signals} ) {
+         foreach( keys %SIG ) {
+            next if m/^__(WARN|DIE)__$/;
+            $SIG{$_} = "DEFAULT" if ref $SIG{$_} eq "CODE";
+         }
+      }
+
+      my $exitvalue = eval { $code->() };
+
+      defined $exitvalue or $exitvalue = -1;
+      _exit( $exitvalue );
+   }
+
+   my $loop = $self->{loop};
+
+   if( defined $params{on_exit} ) {
+      $self->watch_child( $kid => $params{on_exit} );
+   }
+
+   return $kid;
+}
+
 =head1 LOW-LEVEL METHODS
 
 As C<IO::Async::Loop> is an abstract base class, specific subclasses of it are
@@ -1051,11 +1135,11 @@ required API. This method should return the API version that the loop
 implementation supports. The magic constructor will use that class, provided
 it declares a version at least as new as the version documented here.
 
-The current API version is C<0.24>.
+The current API version is C<0.33>.
 
 This method may be implemented using C<constant>; e.g
 
- use constant API_VERSION => '0.24';
+ use constant API_VERSION => '0.33';
 
 =cut
 
