@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Notifier );
 
-our $VERSION = '0.33';
+our $VERSION = '0.34';
 
 use Carp;
 
@@ -25,7 +25,7 @@ C<IO::Async::PID> - event callback on exit of a child process
  use IO::Async::Loop;
  my $loop = IO::Async::Loop->new();
 
- my $kid = $loop->detach_child(
+ my $kid = $loop->fork(
     code => sub {
        print "Child sleeping..\n";
        sleep 10;
